@@ -70,6 +70,7 @@ def _get_model_openai_azure(
     azure_openai_api_key=AZURE_OPENAI_API_KEY,
     azure_openai_endpoint=AZURE_OPENAI_ENDPOINT,
     azure_openai_api_version=AZURE_OPENAI_API_VERSION,
+    http_client=None,
 ):
     assert azure_openai_endpoint, "AZURE_OPENAI_ENDPOINT is not set"
     assert azure_openai_api_key, "AZURE_OPENAI_API_KEY is not set"
@@ -79,6 +80,7 @@ def _get_model_openai_azure(
         azure_endpoint=azure_openai_endpoint,
         api_version=azure_openai_api_version,
         api_key=azure_openai_api_key,
+        http_client=http_client,
     )
     return OpenAIChatModel(model_name=model_name, provider=OpenAIProvider(openai_client=client))
 
@@ -101,7 +103,7 @@ def get_model(model_family=MODEL_FAMILY, model_name=None, http_client=None, **kw
     assert model_family is not None and model_family != "", f"Model family '{model_family}' is not set"
     match model_family:
         case "azure":
-            return _get_model_openai_azure(model_name=model_name or AZURE_MODEL_NAME, **kwargs)
+            return _get_model_openai_azure(model_name=model_name or AZURE_MODEL_NAME, http_client=http_client, **kwargs)
         case "bedrock":
             return _get_model_bedrock(model_name=model_name or BEDROCK_MODEL_NAME, **kwargs)
         case "ollama":
