@@ -3,6 +3,7 @@
 import asyncio
 import logging
 from contextlib import asynccontextmanager
+from dataclasses import dataclass, field
 from datetime import timedelta
 from typing import Any, AsyncGenerator
 
@@ -21,6 +22,8 @@ from pydantic_ai.toolsets.abstract import ToolsetTool
 
 from aixtools.context import SessionIdTuple
 from aixtools.logging.logging_config import get_logger
+from aixtools.server.utils import create_session_headers
+from aixtools.utils.config import MCP_TOOLS_MAX_RETRIES
 
 MCP_TOOL_CACHE_TTL = 300  # 5 minutes
 DEFAULT_MCP_CONNECTION_TIMEOUT = 30
@@ -29,7 +32,6 @@ CACHE_KEY = "TOOL_LIST"
 
 logger = get_logger(__name__)
 
-
 # Default log_handler for MCP clients
 LOGGING_LEVEL_MAP = logging.getLevelNamesMapping()
 
@@ -49,10 +51,18 @@ async def default_mcp_log_handler(message: LogMessage):
     logger.log(level, msg, extra=extra)
 
 
+@dataclass
+class MCPConfig:
+    """Configuration for an MCP server retrieved from config.yaml"""
+
+    url: str
+    read_timeout: float = field(default=DEFAULT_MCP_READ_TIMEOUT)
+
+
 def get_mcp_client(
     url: str | None = None,
     command: str | None = None,
-    args: list[str] = None,
+    args: list[str] | None = None,
     log_handler: callable = default_mcp_log_handler,  # type: ignore
 ) -> MCPServerStreamableHTTP | MCPServerStdio:
     """
@@ -67,63 +77,63 @@ def get_mcp_client(
     if args is None:
         args = []
     if url:
-        return MCPServerStreamableHTTP(url="http://127.0.0.1:8089/mcp/", log_handler=log_handler)
+        return MCPServerStreamableHTTP(url=url, log_handler=log_handler)
     if command:
         return MCPServerStdio(command=command, args=args, log_handler=log_handler)
     raise ValueError("Either url or command must be provided to create MCP client.")
 
 
-def get_mcp_headers(session_id_tuple: SessionIdTuple) -> dict[str, str] | None:
+def get_mcp_servers(
+    mcp_configs: list[MCPConfig],
+    session_id_tuple: SessionIdTuple,
+    auth_token: str = None,
+    max_retries: int = MCP_TOOLS_MAX_RETRIES,
+    *,
+    timeout: float = DEFAULT_MCP_CONNECTION_TIMEOUT,
+):
     """
-    Generate headers for MCP server requests.
+    Create cached MCP server instances with robust error handling and isolation.
 
-    This function creates a dictionary of headers to be used in requests to
-    the MCP servers. If a `user_id` or `session_id` is provided, they are
-    included in the headers.
+    This function creates and returns a list of `CachedMCPServerStreamableHTTP` instances
+    based on the provided URLs. Each server instance includes:
+    - TTL-based caching for tool lists (5 minutes default)
+    - Complete task isolation to prevent cancellation propagation
+    - Comprehensive error handling and fallback mechanisms
+    - Optional user/session headers for request authentication
 
     Args:
-        session_id_tuple (SessionIdTuple): user_id and session_id tuple
+        mcp_configs (list[MCPConfig]): A list of MCP server configurations to use.
+        session_id_tuple (SessionIdTuple): A tuple containing (user_id, session_id).
+        auth_token (str, optional): The authentication token for the user. Defaults to None.
+        timeout (float, optional): Timeout in seconds for MCP server connections.
     Returns:
-        dict[str, str] | None: A dictionary of headers for MCP server requests,
-                               or None if neither user_id nor session_id is
-                               provided. When None is returned, default headers
-                               from the client or transport will be used.
+        list[CachedMCPServerStreamableHTTP]: List of cached MCP server instances with
+                                            isolation and error handling. Each server
+                                            operates independently - failures in one
+                                            server won't affect others.
     """
-    headers = None
-    user_id, session_id = session_id_tuple
-    if session_id or user_id:
-        headers = {}
-        if session_id:
-            headers["session-id"] = session_id
-        if user_id:
-            headers["user-id"] = user_id
-    return headers
+    headers = create_session_headers(session_id_tuple, auth_token)
+    servers = []
+    for config in mcp_configs:
+        server = CachedMCPServerStreamableHTTP(
+            url=config.url, headers=headers, timeout=timeout, read_timeout=config.read_timeout, max_retries=max_retries
+        )
+        logger.info("Using MCP server at %s", config.url)
+        servers.append(server)
+    return servers
 
 
 def get_configured_mcp_servers(
-    session_id_tuple: SessionIdTuple, mcp_urls: list[str], timeout: int = DEFAULT_MCP_CONNECTION_TIMEOUT
+    session_id_tuple: SessionIdTuple,
+    mcp_urls: list[str],
+    *,
+    timeout: int = DEFAULT_MCP_CONNECTION_TIMEOUT,
+    auth_token: str | None = None,
 ):
-    """
-    Retrieve the configured MCP server instances with optional caching.
-
-    Context values `user_id` and `session_id` are included in the headers for each server request.
-
-    Each server is wrapped in a try-except block to isolate them from each other.
-    If one server fails, it won't affect the others.
-
-    Args:
-        session_id_tuple (SessionIdTuple): A tuple containing (user_id, session_id).
-        mcp_urls: (list[str], optional): A list of MCP server URLs to use.
-        timeout (int, optional): Timeout in seconds for MCP server connections. Defaults to 30 seconds.
-    Returns:
-        list[MCPServerStreamableHTTP]: A list of configured MCP server instances. If
-                              neither user_id nor session_id is provided, the
-                              server instances will use default headers defined
-                              by the underlying HTTP implementation.
-    """
-    headers = get_mcp_headers(session_id_tuple)
-
-    return [CachedMCPServerStreamableHTTP(url=url, headers=headers, timeout=timeout) for url in mcp_urls]
+    """Create MCP server instances from a list of URLs."""
+    return get_mcp_servers(
+        [MCPConfig(url=url) for url in mcp_urls], session_id_tuple, timeout=timeout, auth_token=auth_token
+    )
 
 
 class CachedMCPServerStreamableHTTP(MCPServerStreamableHTTP):
@@ -259,7 +269,7 @@ class CachedMCPServerStreamableHTTP(MCPServerStreamableHTTP):
 
         # First, check if we have a valid cached result
         if CACHE_KEY in self._tools_cache:
-            logger.info("Using cached tools for %s", self.url)
+            logger.debug("Using cached tools for %s", self.url)
             return self._tools_cache[CACHE_KEY]
 
         # Create isolated task to prevent cancellation propagation
